#
# gonzui - a source code search engine.
#
# Copyright (C) 2004-2005 Satoru Takabayashi <satoru@namazu.org> 
#     All rights reserved.
#     This is free software with ABSOLUTELY NO WARRANTY.
#
# You can redistribute it and/or modify it under the terms of 
# the GNU General Public License version 2.
#

module Gonzui
  module AutoPack
    module AutoPack
      def [] (key)
        value = super(pack_key(key))
        if value then unpack_value(value) else nil end
      end

      def []= (key, value)
        super(pack_key(key), pack_value(value))
      end

      def delete (key)
        super(pack_key(key))
      end

      def duplicates (key)
        super(pack_key(key)).map {|v| unpack_value(v) }
      end

      def dup_count (key)
        super(pack_key(key))
      end

      def include? (key)
        super(pack_key(key))
      end

      def has_key? (key)
        super(pack_key(key))
      end

      def has_both? (key, value)
        super(pack_key(key), pack_value(value))
      end

      def each
        super {|key, value| yield(unpack_key(key), unpack_value(value)) }
      end

      def each_key
        super {|key| yield(unpack_key(key)) }
      end

      def each_dup (key)
        super(pack_key(key)) {|k, v| yield(unpack_value(k), unpack_value(v)) }
      end
    end

    module PackMethod
      # should be network byte order for sorting values in dbm.
      PACK_FORMAT  = "N"

      def pack_string (string)
        string
      end

      def unpack_string (string)
        string
      end

      def pack_number (n)
        [n].pack(PACK_FORMAT)
      end

      def unpack_number (packed_n)
        packed_n.unpack(PACK_FORMAT).first
      end
    end

    module IdAndId
      include PackMethod
      # no autopack needed
      alias :pack_key     :pack_string
      alias :pack_value   :pack_string
      alias :unpack_key   :unpack_string
      alias :unpack_value :unpack_string
    end

    module IdAndString
      include PackMethod
      # no autopack needed
      alias :pack_key     :pack_string
      alias :pack_value   :pack_string
      alias :unpack_key   :unpack_string
      alias :unpack_value :unpack_string
    end

    module StringAndId
      include PackMethod
      # no autopack needed
      alias :pack_key     :pack_string
      alias :pack_value   :pack_string
      alias :unpack_key   :unpack_string
      alias :unpack_value :unpack_string
    end

    module StringAndString
      include PackMethod
      # no autopack needed
      alias :pack_key     :pack_string
      alias :pack_value   :pack_string
      alias :unpack_key   :unpack_string
      alias :unpack_value :unpack_string
    end

    module StringAndNumber
      include PackMethod
      include AutoPack
      alias :pack_key     :pack_string
      alias :pack_value   :pack_number
      alias :unpack_key   :unpack_string
      alias :unpack_value :unpack_number
    end

    module NumberAndString
      include PackMethod
      include AutoPack
      alias :pack_key     :pack_number
      alias :pack_value   :pack_string
      alias :unpack_key   :unpack_number
      alias :unpack_value :unpack_string
    end
  end
end
