#
# gonzui - a source code search engine.
#
# Copyright (C) 2004-2005 Satoru Takabayashi <satoru@namazu.org> 
#     All rights reserved.
#     This is free software with ABSOLUTELY NO WARRANTY.
#
# You can redistribute it and/or modify it under the terms of 
# the GNU General Public License version 2.
#

require 'bdb'

module Gonzui
  class BDBDBM < AbstractDBM
    include AutoPack::PackMethod

    def initialize (config, read_only = false)
      retried = false
      begin
        @db_mode  = if read_only then BDB::RDONLY else BDB::CREATE end
        env_flags = BDB::CREATE | BDB::INIT_MPOOL | BDB::INIT_LOCK 
        @db_env = BDB::Env.new(config.db_directory, env_flags)
        super(config, read_only)
      rescue BDB::Fatal => e
        raise e if retried
        retried = true
        #
        # "Lock table is out of available locker entries"
        # error ocasionally occurs if a process using
        # Gonzui::BDMDBM is killed by SIGKILL forcibly. In
        # that case, we can remove Berkeley DB Environment
        # to solve the error.
        #
        BDB::Env.remove(config.db_directory, BDB::FORCE)
        retry
      end
    end

    private
    def do_open_db (name, dupsort)
      options = Hash.new
      options["env"] = @db_env
      options["set_flags"] = BDB::DUPSORT if dupsort
      db = BDB::Btree.open(name.to_s, nil, @db_mode, 0644, options)
      db.extend(BDBExtension)
      return db
    end

    public
    def close
      super
      @db_env.close
    end

    def find_all_by_prefix (pattern)
      results = []
      self.word_wordid.each_by_prefix(pattern) {|word, word_id|
        results.concat(collect_all_results(word_id))
      }
      return results
    end
  end

  module BDBExtension
    def each_by_prefix (prefix)
      values = []
      cursor = self.cursor
      begin
        if pair = cursor.set_range(prefix)
          begin
            k, v = pair
            break unless k[0, prefix.length] == prefix
            yield(k, unpack_value(v))
          end while pair = cursor.next
        end
        return values
      ensure
        cursor.close
      end
    end

    def delete_both (key, value)
      cursor = self.cursor
      begin
        pair = cursor.get(BDB::GET_BOTH, pack_key(key), pack_value(value))
        cursor.delete unless pair.nil?
      ensure
        cursor.close
      end
    end

    def get_last_key
      cursor = self.cursor
      begin
        packed_key, packed_value = cursor.last
        if packed_key 
          return unpack_key(packed_key)
        else
          return nil
        end
      ensure
        cursor.close
      end
    end

    def tail (n)
      cursor = self.cursor
      data = []
      begin
        packed_key, packed_value = cursor.last
        while true
          break if data.length >= n or packed_key == nil
          key   = unpack_key(packed_key)
          value = unpack_value(packed_value)
          data.push([key, value])
          packed_key, packed_value = cursor.prev
        end
        return data.reverse!
      ensure
        cursor.close
      end
    end
  end
end
