#
# gonzui - a source code search engine.
#
# Copyright (C) 2004-2005 Satoru Takabayashi <satoru@namazu.org> 
#     All rights reserved.
#     This is free software with ABSOLUTELY NO WARRANTY.
#
# You can redistribute it and/or modify it under the terms of 
# the GNU General Public License version 2.
#

module Gonzui
  QueryValue = Struct.new(:short_name, :default_value, :conversion_method)

  module URLMaker
    ParamTable = {
      :display_language      => QueryValue.new("dl", nil,   :to_s),
      :from                  => QueryValue.new("f",  0,     :to_i),
      :grep_pattern          => QueryValue.new("g",  nil,   :to_s),
      :language              => QueryValue.new("l", "all",  :to_s),
      :basic_query           => QueryValue.new("q",  "",    :to_s),
      :phrase_query          => QueryValue.new("pq", nil,   :to_s),
      :target_type           => QueryValue.new("tt", nil,   :to_s),
      :nresults_per_page     => QueryValue.new("n",  10,    :to_i),
    }

    def decompose_search_query (search_query)
      options = Hash.new
      options[:basic_query] = search_query.string
      options[:language] = search_query.language
      return options
    end

    def get_default_query_value (long_name)
      ParamTable[long_name]
    end

    def get_query_value (query, long_name)
      qv = get_default_query_value(long_name)
      assert(qv)
      value = query[qv.short_name]
      if value and not value.empty?
        return value.send(qv.conversion_method)
      else
        return qv.default_value
      end
    end

    def get_short_name (long_name)
      qv = ParamTable[long_name]
      raise "unknown variable name" if qv.nil?
      return qv.short_name
    end

    def make_advanced_search_url (search_query)
      make_url_general(AdvancedSearchServlet)
    end

    def make_doc_url (path = nil)
      make_url_general(FileHandler, path)
    end

    def make_google_url (search_query)
      sprintf("http://www.google.com/search?q=%s", 
              HTTPUtils.escape(search_query.simplified_string))
    end

    def make_lineno_url (markup_url, lineno)
      sprintf("%s#l%d", markup_url, lineno)
    end

    def make_markup_url (path = "", search_query = nil, options = {})
      path = if path.empty?
               MarkupServlet.mount_point
             else
               sprintf("%s/%s", 
                       MarkupServlet.mount_point,
                       HTTPUtils.escape(path))
             end
      if search_query
        decompose_search_query(search_query).each {|k, v|
          options[k] = v
        }
      end
      return make_url_with_options(path, options)
    end

    def make_search_url (search_query, options = {})
      options.merge!(decompose_search_query(search_query))
      make_url_with_options(SearchServlet.mount_point, options)
    end

    def make_search_url_by_query_string (query_string)
      make_url_with_options(SearchServlet.mount_point, 
                            :basic_query => query_string)
    end

    def make_source_url (path = nil)
      make_url_general(SourceServlet, path)
    end

    def make_stat_url (path = nil)
      make_url_general(StatisticsServlet, path)
    end

    def make_url_general (klass, path = nil)
      if path
        return sprintf("%s/%s", klass.mount_point, HTTPUtils.escape(path))
      else
        return klass.mount_point
      end
    end

    def make_url_with_options (path, options)
      params = []
      options.each {|name, value|
        next if value.nil?
        qv = get_default_query_value(name)
        if qv and value != qv.default_value
          param = sprintf("%s=%s", qv.short_name, HTTPUtils.escape(value.to_s))
          params.push(param)
        end
      }
      url = path
      url += "?" + params.join("&") unless params.empty?
      return url
    end
  end
end
